@@ -11,10 +11,7 @@ use PYX qw(end_element);
 use PYX::Parser;
 
 # Version.
-our $VERSION = 0.02;
-
-# Global variables.
-use vars qw($stack $rules $flush_stack);
+our $VERSION = 0.03;
 
 # Constructor.
 sub new {
@@ -46,19 +43,15 @@ sub new {
 			'final' => \&_final,
 			'start_element' => \&_start_element,
 		},
+		'non_parser_options' => {
+			'flush_stack' => $self->{'flush_stack'},
+			'rules' => $self->{'rules'},
+			'stack' => [],
+		},
 		'output_handler' => $self->{'output_handler'},
 		'output_rewrite' => 1,
 	);
 
-	# Tag values.
-	$stack = [];
-
-	# Rules.
-	$rules = $self->{'rules'};
-
-	# Flush stack.
-	$flush_stack = $self->{'flush_stack'};
-
 	# Object.
 	return $self;
 }
@@ -88,6 +81,8 @@ sub parse_handler {
 sub _start_element {
 	my ($pyx_parser, $tag) = @_;
 	my $out = $pyx_parser->{'output_handler'};
+	my $rules = $pyx_parser->{'non_parser_options'}->{'rules'};
+	my $stack = $pyx_parser->{'non_parser_options'}->{'stack'};
 	if (exists $rules->{'*'}) {
 		foreach my $tmp (@{$rules->{'*'}}) {
 			if (@{$stack} > 0 && lc($stack->[-1]) eq $tmp) {
@@ -110,22 +105,17 @@ sub _start_element {
 # Add implicit end_element.
 sub _end_element_simple {
 	my $pyx_parser = shift;
-
-	# Output handler.
+	my $rules = $pyx_parser->{'non_parser_options'}->{'rules'};
+	my $stack = $pyx_parser->{'non_parser_options'}->{'stack'};
 	my $out = $pyx_parser->{'output_handler'};
-
-	# Process.
 	if (exists $rules->{'*'}) {
 		foreach my $tmp (@{$rules->{'*'}}) {
-			if (lc $stack->[-1] eq $tmp) {
+			if (@{$stack} && lc $stack->[-1] eq $tmp) {
 				print {$out} end_element(pop @{$stack}), "\n";
 			}
 		}
 	}
-
-	# Print line.
 	print {$out} $pyx_parser->line, "\n";
-
 	return;
 }
 
@@ -133,6 +123,8 @@ sub _end_element_simple {
 sub _end_element {
 	my ($pyx_parser, $tag) = @_;
 	my $out = $pyx_parser->{'output_handler'};
+	my $rules = $pyx_parser->{'non_parser_options'}->{'rules'};
+	my $stack = $pyx_parser->{'non_parser_options'}->{'stack'};
 	if (exists $rules->{'*'}) {
 		foreach my $tmp (@{$rules->{'*'}}) {
 			if (lc($tag) ne $tmp && lc($stack->[-1]) eq $tmp) {
@@ -158,11 +150,12 @@ sub _end_element {
 # Process final.
 sub _final {
 	my $pyx_parser = shift;
+	my $stack = $pyx_parser->{'non_parser_options'}->{'stack'};
 	my $out = $pyx_parser->{'output_handler'};
 	if (@{$stack} > 0) {
 
 		# If set, than flush stack.
-		if ($flush_stack) {
+		if ($pyx_parser->{'non_parser_options'}->{'flush_stack'}) {
 			foreach my $tmp (reverse @{$stack}) {
 				print {$out} end_element($tmp), "\n";
 			}
@@ -216,11 +209,12 @@ PYX::XMLNorm - Processing PYX data or file and do XML normalization.
  XML normalization rules.
  Parameter is required.
  Format of rules is:
- outer element => list of inner elements.
+ Outer element => list of inner elements.
  e.g.
  {
          'middle' => ['end'],
  },
+ Outer element can be '*'.
  Default value is {}.
 
 =back
@@ -299,12 +293,6 @@ L<PYX::Parser>.
 
 =head1 SEE ALSO
 
-L<PYX>,
-L<PYX::GraphViz>,
-L<PYX::Hist>,
-L<PYX::Parser>,
-L<PYX::Stack>,
-L<PYX::Utils>,
 L<Task::PYX>.
 
 =head1 REPOSITORY
@@ -322,6 +310,6 @@ Michal Špaček L<skim@cpan.org>.
 
 =head1 VERSION
 
-0.02
+0.03
 
 =cut
